# Personal Humidity Controller and Logger
# A Debby Solutions program
# www.debbysolutions.com
# 1/5/2022
#
from guizero import *                       #for GUI creation
import RPi.GPIO as GPIO                     #for GPIO control
import Adafruit_DHT as dht                  #for DHT-22 control                
from datetime import *                      #will use for getting current date and time
from time import sleep                      #will use sleep to set delay

#Suppress the GPIO warnings
GPIO.setwarnings(False)

#Set GPIO23 as an output to control the AC Power Relay Module
GPIO.setmode(GPIO.BCM)                      #Use GPIO pin numbering
GPIO.setup(23,GPIO.OUT)                     #set GPIO23 as output
#Set DHT variable to indicate that GPIO17 will be used for DHT-22 sensor
DHT = 17

#set up program variables
count = 0
humidifierState = "OFF"
desiredHumidity = 70                        #initial humidity set point in %
desiredVariance = 1.0                       #humidifier will be controlled within 1/2 of this humidity range
currentDate = date.today()                  #the current date will be used as log file name

#Create file and write out header row in new CSV file
filename = str(currentDate) + ".csv"
csv = open(filename, 'w')
csv.write("Timestamp, Temperature, Humidity, Heater State\n")
csv.close()

#function to read data from the DHT-22 sensor to turn humidifier ON/OFF and also write value out to log file
def read_hum():
    global humidifierState, filename, currentDate, count
    today = date.today()
    #start a new file if we are on a different date
    if (today != currentDate):                                  #start a new file if we are on a different date
        filename = str(today) + ".csv"
        currentDate = today
        csv = open(filename, 'w')
        csv.write("Timestamp, Temperature, Humidity, Humidifier State\n")
        csv.close()
    #Read Temp and Humidity from DHT22
    h,t = dht.read_retry(dht.DHT22, DHT)                        #read humidity and temperature from DHT-22 sensor
    ft = round((t*9/5)+32,1)                                    #convert temperature from C to F
    rh = round(h,1)                                             #round the humidity value to 1 place
    if (h < desiredHumidity - desiredVariance/2):               #if humidity falls below desired level
        GPIO.output(23,GPIO.HIGH)                                               #turn humidifier ON
        humidifierState = "ON"
    if (h > desiredHumidity + desiredVariance/2):               #if humidity rises above desired level
        GPIO.output(23,GPIO.LOW)                                                #turn humidifier OFF
        humidifierState = "OFF"
    if (count >= 30):                                            #if number of times we have read from DHT-22 is greater than 30
        temp_c = str(round(ft,1))                                               #convert temperature value to string
        hum_c = str(round(h,1))                                                 #convert humidity value to string
        now = datetime.now()                                                    #get current time, and
        entry = now.strftime("%H:%M")                                           #convert to hour and minute format
        entry = entry + "," + temp_c + "," + hum_c + "," + humidifierState + "\n"   #create complete string, and
        #append log entry to file
        csv = open(filename, 'a')                                               #append to log file
        try:
            csv.write(entry)
        finally:
            csv.close()
        count = 0
    else:                                                       #if less than 30 times,
        count = count + 1                                                       #increment count
    return rh                                                   #return the acquired humidity rounded to 1 place of decimal

#function to update the desired and current temperature values on the GUI
def update_label():
    currentHumidity.value = read_hum()                           #call the read_temp() function to acquire the humidity in %
    desiredHumidity.value = desiredHumidity                      #update desired temperature since that could have been changed
    # recursive call
    currentHumidity.after(1000, update_label)                    #will call itself after 1000 mS recursively to keep updating GUI

#function to increase desired temperature by one degree
def upHumidity():
    global desiredHumidity
    desiredHumidity = desiredHumidity + 1

#function to decrease desired temperature by one degree
def downHumidity():
    global desiredHumidity
    desiredTemperature = desiredHumidity - 1

#Main execution code to set up the GUI window and displaying it
if __name__=='__main__':
    #instantiate a GUI window with title, size and format
    app = App(title="dTemp - a Debby Solution",
              height=180,
              width=450,
              layout='grid')
    #define the GUI window content
    DesiredTitle = Text(app, 'Desired Humidity', grid=[0,0])                             #set up title of Desired Humidity and location, and
    desiredTemp = Text(app, "xx", size=80 , color="gray", grid=[0,1])                       #set "xx" as placeholder with desired size and location
    upTempButton = PushButton(app, command=upHumidity, text=' '+chr(94)+' ', grid=[1,0])        #set up ^ button and will call upHumidity function when touched
    downTempButton = PushButton(app, command=downHumidity, text=' '+chr(118)+' ', grid=[1,1])   #set up v button and will call downHumidity function when touched
    Space1 = Text(app, '   ', grid=[2,0])                                                   #add some space between buttons and Current Humidity
    Space2 = Text(app, '   ', grid=[2,1])                                                   #add some space between buttons and Current Humidity
    CurrentTitle = Text(app, 'Current Humidity', grid=[3,0])                             #set up title of Current Humidity and location, and
    currentHumidity = Text(app, "xx", size=80, grid=[3,1])                                      #set "xx" as placeholder with desired size location
    currentHumidity.after(1000, update_label)
    #display the GUI window
    app.display()
